#!/usr/local/bin/python

"""
Copyright (c) 2004, Jan Stocker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Jan Stocker nor the names of its contributors may
      be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
"""

from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.platypus import Paragraph, Table, SimpleDocTemplate
from reportlab.platypus import Table, TableStyle
from reportlab.lib import colors
from reportlab.lib.units import mm

import cgi

from battle import ThudMove, ThudBattle

from Tkinter import *
import tkColorChooser
import tkFileDialog
import tkSimpleDialog
import tkMessageBox

import ConfigParser



def RGBToString(rgbColor):
    hexcolor = '#%02x%02x%02x' % ( int(rgbColor[0] * 255), int(rgbColor[1] * 255), int(rgbColor[2] * 255))
    return hexcolor

def StringToRGB(strColor):
    strColor = strColor[0] == '#' and strColor[1:] or strColor
    return (int(strColor[:2],16) / 255.0, int(strColor[2:4],16) / 255.0, int(strColor[4:],16) / 255.0)



class ThudPDFReportConfig(object):
    bgcolorFieldWhite = (1,1,1)
    textcolorFieldWhite = (0,0,0)
    bgcolorFieldBlack = (0.5,0.5,0.5)
    textcolorFieldBlack = (0,0,0)
    colorGrid = (0,0,0)
    linewidthGrid = 0.25
    colorMovebox = (0,0,0)
    linewidthMovebox = 1.75
    textcolorCoords = (0.7,0.7,0.7)
    
    flagCoordsAround = True
    flagCoordsInline = True
    flagMovebox = True
    flagMarkThudded = True
    flagOnlySaved = True

    sizeModeOne = [45, 80, 55]
    sizeModeTwo = [40, 55, 40, 55]

    def load(self, filename, section="ThudPDFReport"):
        config = ConfigParser.ConfigParser()
        config.read(filename)
        try:
            self.flagCoordsAround = config.getboolean(section, 'flagCoordsAround')
            self.flagCoordsInline = config.getboolean(section, 'flagCoordsInline')
            self.flagMovebox = config.getboolean(section, 'flagMovebox')
            self.flagMarkThudded = config.getboolean(section, 'flagMarkThudded')
            self.flagOnlySaved = config.getboolean(section, 'flagOnlySaved')
            self.bgcolorFieldWhite = StringToRGB(config.get(section, 'bgcolorFieldWhite'))
            self.textcolorFieldWhite = StringToRGB(config.get(section, 'textcolorFieldWhite'))
            self.bgcolorFieldBlack = StringToRGB(config.get(section, 'bgcolorFieldBlack'))
            self.textcolorFieldBlack = StringToRGB(config.get(section, 'textcolorFieldBlack'))
            self.colorGrid = StringToRGB(config.get(section, 'colorGrid'))
            self.linewidthGrid = config.getfloat(section, 'linewidthGrid')
            self.colorMovebox = StringToRGB(config.get(section, 'colorMovebox'))
            self.linewidthMovebox = config.getfloat(section, 'linewidthMovebox')
            self.textcolorCoords = StringToRGB(config.get(section, 'textcolorCoords'))
            self.sizeModeOne[0] = config.getfloat(section, 'sizeModeOneCol1')
            self.sizeModeOne[1] = config.getfloat(section, 'sizeModeOneCol3')
            self.sizeModeOne[2] = config.getfloat(section, 'sizeModeOneCol3')
            self.sizeModeTwo[0] = config.getfloat(section, 'sizeModeTwoCol1')
            self.sizeModeTwo[1] = config.getfloat(section, 'sizeModeTwoCol3')
            self.sizeModeTwo[2] = config.getfloat(section, 'sizeModeTwoCol3')
            self.sizeModeTwo[3] = config.getfloat(section, 'sizeModeTwoCol4')
        except:
            pass 

    def save(self, filename, section="ThudPDFReport"):
        config = ConfigParser.ConfigParser()
        config.read(filename)

        if not config.has_section(section):
            config.add_section(section)
        try:
            config.set(section, 'flagCoordsAround', self.flagCoordsAround) 
            config.set(section, 'flagCoordsInline', self.flagCoordsInline) 
            config.set(section, 'flagMovebox', self.flagMovebox) 
            config.set(section, 'flagMarkThudded', self.flagMarkThudded) 
            config.set(section, 'flagOnlySaved', self.flagOnlySaved) 
            config.set(section, 'bgcolorFieldWhite',RGBToString(self.bgcolorFieldWhite)) 
            config.set(section, 'textcolorFieldWhite',RGBToString(self.textcolorFieldWhite))
            config.set(section, 'bgcolorFieldBlack',RGBToString(self.bgcolorFieldBlack))
            config.set(section, 'textcolorFieldBlack',RGBToString(self.textcolorFieldBlack))
            config.set(section, 'colorGrid',RGBToString(self.colorGrid)) 
            config.set(section, 'linewidthGrid', self.linewidthGrid) 
            config.set(section, 'colorMovebox',RGBToString(self.colorMovebox)) 
            config.set(section, 'linewidthMovebox', self.linewidthMovebox) 
            config.set(section, 'textcolorCoords',RGBToString(self.textcolorCoords))
            config.set(section, 'sizeModeOneCol1', self.sizeModeOne[0]) 
            config.set(section, 'sizeModeOneCol3', self.sizeModeOne[1]) 
            config.set(section, 'sizeModeOneCol3', self.sizeModeOne[2]) 
            config.set(section, 'sizeModeTwoCol1', self.sizeModeTwo[0]) 
            config.set(section, 'sizeModeTwoCol3', self.sizeModeTwo[1]) 
            config.set(section, 'sizeModeTwoCol3', self.sizeModeTwo[2]) 
            config.set(section, 'sizeModeTwoCol4', self.sizeModeTwo[3]) 
            fp = open(filename, 'w+b')
            config.write(fp)
            fp.close()
        except:
            showwarning('Save options failed', \
                        'Saving the configuration failed. The file "'+filename+'" or folder may be read-only or corrupt.')

    def showEditDialog(self, parent):
        dlg = ThudPDFReportConfigDialog(self, parent)
        pass






class ThudPDFReportConfigDialog(tkSimpleDialog.Dialog):
    def __init__(self, cfg, parent):
        self.cfg = cfg
        tkSimpleDialog.Dialog.__init__(self, parent, title="ThudReport - Options")

    def body(self, master):
        self.flagCoordsAround = IntVar()
        self.flagCoordsInline = IntVar()
        self.flagMovebox = IntVar()
        self.flagMarkThudded = IntVar()
        self.flagOnlySaved = IntVar()

        self.strBGColorFieldWhite = RGBToString(self.cfg.bgcolorFieldWhite)
        self.strTextcolorFieldWhite = RGBToString(self.cfg.textcolorFieldWhite)
        self.strBGColorFieldBlack = RGBToString(self.cfg.bgcolorFieldBlack)
        self.strTextcolorFieldBlack = RGBToString(self.cfg.textcolorFieldBlack)
        self.strColorGrid = RGBToString(self.cfg.colorGrid)
        self.strColorMovebox = RGBToString(self.cfg.colorMovebox)
        self.strTextcolorCoords = RGBToString(self.cfg.textcolorCoords)

        self.checkboxCoordsAround = Checkbutton(master, text="Draw coords outside board", variable=self.flagCoordsAround)
        self.checkboxCoordsInline = Checkbutton(master, text="Draw coords inside board", variable=self.flagCoordsInline)
        self.checkboxMovebox = Checkbutton(master, text="Draw a box for start and end move position", variable=self.flagMovebox)
        self.checkboxMarkThudded = Checkbutton(master, text="Draw thudded pieces", variable=self.flagMarkThudded)
        self.checkboxOnlySaved = Checkbutton(master, text="Draw only saved moves", variable=self.flagOnlySaved)

        self.checkboxCoordsAround.grid(row=0, column=0, columnspan=5, sticky=W)
        self.checkboxCoordsInline.grid(row=1, column=0, columnspan=5, sticky=W)
        self.checkboxMovebox.grid(row=2, column=0, columnspan=5, sticky=W)
        self.checkboxMarkThudded.grid(row=3, column=0, columnspan=5, sticky=W)
        self.checkboxOnlySaved.grid(row=4, column=0, columnspan=5, sticky=W)

        self.labelSizeModeOne = Label(master, text="Mode 1 / column sizes:")
        self.editSizeModeOneCol1 = Entry(master, width=5)
        self.editSizeModeOneCol2 = Entry(master, width=5)
        self.editSizeModeOneCol3 = Entry(master, width=5)

        self.labelSizeModeOne.grid(row=6, column=0, sticky=E)
        self.editSizeModeOneCol1.grid(row=6, column=1)
        self.editSizeModeOneCol2.grid(row=6, column=2)
        self.editSizeModeOneCol3.grid(row=6, column=3)

        self.labelSizeModeTwo = Label(master, text="Mode 2 / column sizes:")
        self.editSizeModeTwoCol1 = Entry(master, width=5)
        self.editSizeModeTwoCol2 = Entry(master, width=5)
        self.editSizeModeTwoCol3 = Entry(master, width=5)
        self.editSizeModeTwoCol4 = Entry(master, width=5)

        self.labelSizeModeTwo.grid(row=7, column=0, sticky=E)
        self.editSizeModeTwoCol1.grid(row=7, column=1)
        self.editSizeModeTwoCol2.grid(row=7, column=2)
        self.editSizeModeTwoCol3.grid(row=7, column=3)
        self.editSizeModeTwoCol4.grid(row=7, column=4)

        self.labelBGColorFieldWhite = Label(master, text="White field background:")
        self.buttonBGColorFieldWhite= Button(master, bg=self.strBGColorFieldWhite, activebackground=self.strBGColorFieldWhite, command=self.ChooseColorBGColorFieldWhite, width=2)
        
        self.labelBGColorFieldWhite.grid(row=9, column=0, sticky=E)
        self.buttonBGColorFieldWhite.grid(row=9, column=1, columnspan=4, sticky=W)

        self.labelTextcolorFieldWhite = Label(master, text="White field text color:")
        self.buttonTextcolorFieldWhite= Button(master, bg=self.strTextcolorFieldWhite, activebackground=self.strTextcolorFieldWhite, command=self.ChooseColorTextcolorFieldWhite, width=2)

        self.labelTextcolorFieldWhite.grid(row=10, column=0, sticky=E)
        self.buttonTextcolorFieldWhite.grid(row=10, column=1, columnspan=4, sticky=W)

        self.labelBGColorFieldBlack = Label(master, text="Black field background:")
        self.buttonBGColorFieldBlack= Button(master, bg=self.strBGColorFieldBlack, activebackground=self.strBGColorFieldBlack, command=self.ChooseColorBGColorFieldBlack, width=2)
        
        self.labelBGColorFieldBlack.grid(row=11, column=0, sticky=E)
        self.buttonBGColorFieldBlack.grid(row=11, column=1, columnspan=4, sticky=W)

        self.labelTextcolorFieldBlack = Label(master, text="Black field text color:")
        self.buttonTextcolorFieldBlack= Button(master, bg=self.strTextcolorFieldBlack, activebackground=self.strTextcolorFieldBlack, command=self.ChooseColorTextcolorFieldBlack, width=2)

        self.labelTextcolorFieldBlack.grid(row=12, column=0, sticky=E)
        self.buttonTextcolorFieldBlack.grid(row=12, column=1, columnspan=4, sticky=W)

        self.labelColorGrid = Label(master, text="Grid color:")
        self.buttonColorGrid= Button(master, bg=self.strColorGrid, activebackground=self.strColorGrid, command=self.ChooseColorColorGrid, width=2)
        self.labelLinewidthGrid = Label(master, text="Grid line width:")
        self.editLinewidthGrid = Entry(master, width=5)

        self.labelColorGrid.grid(row=13, column=0, sticky=E)
        self.buttonColorGrid.grid(row=13, column=1, columnspan=4, sticky=W)
        self.labelLinewidthGrid.grid(row=14, column=0, sticky=E)
        self.editLinewidthGrid.grid(row=14, column=1, columnspan=4, sticky=W)

        self.labelColorMovebox = Label(master, text="Movebox color:")
        self.buttonColorMovebox= Button(master, bg=self.strColorMovebox, activebackground=self.strColorMovebox, command=self.ChooseColorColorMovebox, width=2)
        self.labelLinewidthMovebox = Label(master, text="Movebox line width:")
        self.editLinewidthMovebox = Entry(master, width=5)

        self.labelColorMovebox.grid(row=15, column=0, sticky=E)
        self.buttonColorMovebox.grid(row=15, column=1, columnspan=4, sticky=W)
        self.labelLinewidthMovebox.grid(row=16, column=0, sticky=E)
        self.editLinewidthMovebox.grid(row=16, column=1, columnspan=4, sticky=W)

        self.labelTextcolorCoords = Label(master, text="Inline coordinates color:")
        self.buttonTextcolorCoords= Button(master, bg=self.strTextcolorCoords, activebackground=self.strTextcolorCoords, command=self.ChooseColorTextcolorCoords, width=2)
        self.labelTextcolorCoords.grid(row=17, column=0, sticky=E)
        self.buttonTextcolorCoords.grid(row=17, column=1, columnspan=4, sticky=W)
        


        if self.cfg.flagCoordsAround:
            self.checkboxCoordsAround.select()
        if self.cfg.flagCoordsInline:
            self.checkboxCoordsInline.select()
        if self.cfg.flagMovebox:
            self.checkboxMovebox.select()
        if self.cfg.flagMarkThudded:
            self.checkboxMarkThudded.select()
        if self.cfg.flagOnlySaved:
            self.checkboxOnlySaved.select()

        #Edit fields content
        self.editSizeModeOneCol1.insert(END, str(self.cfg.sizeModeOne[0]))
        self.editSizeModeOneCol2.insert(END, str(self.cfg.sizeModeOne[1]))
        self.editSizeModeOneCol3.insert(END, str(self.cfg.sizeModeOne[2]))

        self.editSizeModeTwoCol1.insert(END, str(self.cfg.sizeModeTwo[0]))
        self.editSizeModeTwoCol2.insert(END, str(self.cfg.sizeModeTwo[1]))
        self.editSizeModeTwoCol3.insert(END, str(self.cfg.sizeModeTwo[2]))
        self.editSizeModeTwoCol4.insert(END, str(self.cfg.sizeModeTwo[3]))

        self.editLinewidthGrid.insert(END, str(self.cfg.linewidthGrid))
        self.editLinewidthMovebox.insert(END, str(self.cfg.linewidthMovebox))

        return self.checkboxCoordsAround # initial focus

    def validate(self):
        try:
            sizeModeOne = [0,0,0]
            sizeModeOne[0] = float(self.editSizeModeOneCol1.get())
            sizeModeOne[1] = float(self.editSizeModeOneCol2.get())
            sizeModeOne[2] = float(self.editSizeModeOneCol3.get())

            sizeModeTwo = [0,0,0,0]
            sizeModeTwo[0] = float(self.editSizeModeTwoCol1.get())
            sizeModeTwo[1] = float(self.editSizeModeTwoCol2.get())
            sizeModeTwo[2] = float(self.editSizeModeTwoCol3.get())
            sizeModeTwo[3] = float(self.editSizeModeTwoCol4.get())

            flagHasNegative = False
            nWidthModeOne = 0
            for i in range(3):
                nWidthModeOne = nWidthModeOne + sizeModeOne[i]
                if sizeModeOne[i] < 0:
                    flagHasNegative = True
                    break

            nWidthModeTwo = 0
            for i in range(4):
                nWidthModeTwo = nWidthModeTwo + sizeModeTwo[i]
                if sizeModeTwo[i] < 0:
                    flagHasNegative = True
                    break
            
            if flagHasNegative:
                tkMessageBox.showwarning(
                        "Bad input",
                        "The column sizes must only contain positive numbers!"
                        )
                return 0
            if nWidthModeOne > 200 or nWidthModeTwo > 200:
                tkMessageBox.showwarning(
                        "Bad input",
                        "The added column sizes must not exceed 200 mm (20 cm)!"
                        )
                return 0
        except ValueError:
            tkMessageBox.showwarning(
                    "Bad input",
                    "The column sizes must only contain decimal numbers!"
                    )
            return 0

        try:
            linewidthGrid = float(self.editLinewidthGrid.get())
            linewidthMovebox = float(self.editLinewidthMovebox.get())
            
            if linewidthGrid <= 0 or linewidthMovebox <= 0:
                tkMessageBox.showwarning(
                        "Bad input",
                        "The line width must be greater than 0!"
                        )
                return 0
                
        except:
            tkMessageBox.showwarning(
                    "Bad input",
                    "The line width must only contain decimal numbers!"
                    )
            return 0

        return 1
    
    def apply(self):
        self.cfg.flagCoordsAround = (self.flagCoordsAround.get() == 1)
        self.cfg.flagCoordsInline = (self.flagCoordsInline.get() == 1)
        self.cfg.flagMovebox = (self.flagMovebox.get() == 1)
        self.cfg.flagMarkThudded = (self.flagMarkThudded.get() == 1)
        self.cfg.flagOnlySaved = (self.flagOnlySaved.get() == 1)

        self.cfg.sizeModeOne[0] = float(self.editSizeModeOneCol1.get())
        self.cfg.sizeModeOne[1] = float(self.editSizeModeOneCol2.get())
        self.cfg.sizeModeOne[2] = float(self.editSizeModeOneCol3.get())

        self.cfg.sizeModeTwo[0] = float(self.editSizeModeTwoCol1.get())
        self.cfg.sizeModeTwo[1] = float(self.editSizeModeTwoCol2.get())
        self.cfg.sizeModeTwo[2] = float(self.editSizeModeTwoCol3.get())
        self.cfg.sizeModeTwo[3] = float(self.editSizeModeTwoCol4.get())

        self.cfg.linewidthGrid = float(self.editLinewidthGrid.get())
        self.cfg.linewidthMovebox = float(self.editLinewidthMovebox.get())

        self.cfg.bgcolorFieldWhite = StringToRGB(self.strBGColorFieldWhite)
        self.cfg.textcolorFieldWhite = StringToRGB(self.strTextcolorFieldWhite)
        self.cfg.bgcolorFieldBlack = StringToRGB(self.strBGColorFieldBlack)
        self.cfg.textcolorFieldBlack = StringToRGB(self.strTextcolorFieldBlack)
        self.cfg.colorGrid = StringToRGB(self.strColorGrid)
        self.cfg.colorMovebox = StringToRGB(self.strColorMovebox)
        self.cfg.textcolorCoords = StringToRGB(self.strTextcolorCoords)

    def ChooseColorBGColorFieldWhite(self):
        (rgb, tkcolor) = tkColorChooser.askcolor(initialcolor=self.strBGColorFieldWhite)
        if tkcolor:
            self.strBGColorFieldWhite = str(tkcolor)
            self.buttonBGColorFieldWhite.configure(bg=self.strBGColorFieldWhite, activebackground=self.strBGColorFieldWhite)

    def ChooseColorTextcolorFieldWhite(self):
        (rgb, tkcolor) = tkColorChooser.askcolor(initialcolor=self.strTextcolorFieldWhite)
        if tkcolor:
            self.strTextcolorFieldWhite = str(tkcolor)
            self.buttonTextcolorFieldWhite.configure(bg=self.strTextcolorFieldWhite, activebackground=self.strTextcolorFieldWhite)

    def ChooseColorBGColorFieldBlack(self):
        (rgb, tkcolor) = tkColorChooser.askcolor(initialcolor=self.strBGColorFieldBlack)
        if tkcolor:
            self.strBGColorFieldBlack = str(tkcolor)
            self.buttonBGColorFieldBlack.configure(bg=self.strBGColorFieldBlack, activebackground=self.strBGColorFieldBlack)

    def ChooseColorTextcolorFieldBlack(self):
        (rgb, tkcolor) = tkColorChooser.askcolor(initialcolor=self.strTextcolorFieldBlack)
        if tkcolor:
            self.strTextcolorFieldBlack = str(tkcolor)
            self.buttonTextcolorFieldBlack.configure(bg=self.strTextcolorFieldBlack, activebackground=self.strTextcolorFieldBlack)

    def ChooseColorColorGrid(self):
        (rgb, tkcolor) = tkColorChooser.askcolor(initialcolor=self.strColorGrid)
        if tkcolor:
            self.strColorGrid = str(tkcolor)
            self.buttonColorGrid.configure(bg=self.strColorGrid, activebackground=self.strColorGrid)

    def ChooseColorColorMovebox(self):
        (rgb, tkcolor) = tkColorChooser.askcolor(initialcolor=self.strColorMovebox)
        if tkcolor:
            self.strColorMovebox = str(tkcolor)
            self.buttonColorMovebox.configure(bg=self.strColorMovebox, activebackground=self.strColorMovebox)

    def ChooseColorTextcolorCoords(self):
        (rgb, tkcolor) = tkColorChooser.askcolor(initialcolor=self.strTextcolorCoords)
        if tkcolor:
            self.strTextcolorCoords = str(tkcolor)
            self.buttonTextcolorCoords.configure(bg=self.strTextcolorCoords, activebackground=self.strTextcolorCoords)

class ThudPDFReport(object):

    modeOne = 1
    modeTwo = 2

    cfg = ThudPDFReportConfig()
    

    def __init__(self):
        pass

    def createBoard(self, thudbattle, index, sizeBoard):
        sizePlace      = sizeBoard * mm / 15.0
        fontsizePieces = (sizeBoard * 9.0) / 75.0; 
        fontsizeCoords = fontsizePieces * 0.50
        nCoords        = self.cfg.flagCoordsAround and 1 or 0
        coordnamesX = ['A','B','C','D','E','F','G','H','J','K','L','M','N','O','P']
        data = []


        for y in range(15 + (nCoords*2)):
            data.append(["" for x in range(15 + (nCoords*2))])

        tabstyle = TableStyle([ ('ALIGN',(0,0),(-1,-1), 'CENTER'),
                                ('VALIGN',(0,0),(-1,-1), 'BOTTOM'),
                                ('FONTSIZE',(0,0), (-1,-1), fontsizePieces),
                                ('LEADING',(0,0), (-1,-1), fontsizePieces),
                                ])
        parstyle = ParagraphStyle(name='Coords',
                                  fontName='Times-Roman',
                                  fontSize=fontsizeCoords,
                                  leading=fontsizeCoords*0.75,
                                  spaceBefore=0,
                                  spaceAfter=0,
                                  textColor= colors.Color(self.cfg.textcolorCoords[0],self.cfg.textcolorCoords[1],self.cfg.textcolorCoords[2]))

        fields = [5,7,9,11,13,15,15,15,15,15,13,11,9,7,5]
        for y in range(15):
            for x in range(fields[y]):
                xpos = int(x+((15 - fields[y]) / 2))
                flagFieldWhite = ((xpos % 2) ^ (y % 2)) == 1
                xpos = xpos + nCoords
                ypos = y + nCoords
                if (flagFieldWhite):
                    tabstyle.add('BACKGROUND', (xpos, ypos), (xpos, ypos), self.cfg.bgcolorFieldBlack)
                    tabstyle.add('TEXTCOLOR', (xpos, ypos), (xpos, ypos), self.cfg.textcolorFieldBlack)
                else:
                    tabstyle.add('BACKGROUND', (xpos, ypos), (xpos, ypos), self.cfg.bgcolorFieldWhite)
                    tabstyle.add('TEXTCOLOR', (xpos, ypos), (xpos, ypos), self.cfg.textcolorFieldWhite)
                tabstyle.add('BOX', (xpos,ypos), (xpos,ypos), self.cfg.linewidthGrid, self.cfg.colorGrid)
                if (self.cfg.flagCoordsInline):
                    data[ypos][xpos] = Paragraph("%s%d" % (coordnamesX[xpos-nCoords], y+1),parstyle)

        if (nCoords != 0):
            for x in range(len(coordnamesX)):
                data[0][x+1] = coordnamesX[x]
                data[16][x+1] = coordnamesX[x]
            for y in range(15):
                data[y+1][0] = ("%d" % (y+1))
                data[y+1][16] = ("%d" % (y+1))

        data[7+nCoords][7+nCoords] = "O";

        position = thudbattle.get_position(index)
        for piece in (thudbattle.dwarf, thudbattle.troll):
            for coord in position[piece]:
                posx = thudbattle.mapx[coord[0]] + nCoords
                posy = int(coord[1:])-1 + nCoords
                data[posy][posx] =  piece == 0 and "d" or "T"
        
        if self.cfg.flagMovebox and index >= 0:
            posx = thudbattle.mapx[thudbattle.moves[index].from_pos[0]] + nCoords
            posy = int(thudbattle.moves[index].from_pos[1:])-1 + nCoords
            tabstyle.add('BOX',(posx,posy),(posx,posy), self.cfg.linewidthMovebox, self.cfg.colorMovebox);
        
            posx = thudbattle.mapx[thudbattle.moves[index].to_pos[0]] + nCoords
            posy = int(thudbattle.moves[index].to_pos[1:])-1 + nCoords
            tabstyle.add('BOX',(posx,posy),(posx,posy), self.cfg.linewidthMovebox, self.cfg.colorMovebox);
        
        if self.cfg.flagMarkThudded and index >=0:
            for thudded in thudbattle.moves[index].thud_list:
                if thudded != thudbattle.moves[index].to_pos:
                    posx = thudbattle.mapx[thudded[0]] + nCoords
                    posy = int(thudded[1:])-1 + nCoords
                    data[posy][posx] = "x"

        tab = Table(data, (15 + (nCoords*2)) * [sizePlace], (15 + (nCoords*2)) * [sizePlace])
        tab.setStyle(tabstyle)

        return tab

    def createPDF(self, filenameThud, filenamePDF, mode, header=''):
        styleSheet = getSampleStyleSheet()
        styleNormal = styleSheet['Normal']

        arrPDF    = []
        arrBattle = []

        thudbattle = ThudBattle()
        if (thudbattle.load(filenameThud) == False):
            return False

        if header:
            arrPDF.append(Paragraph(cgi.escape(header), styleNormal))

        countMoves = self.cfg.flagOnlySaved and (thudbattle.saved_index + 1) or len(thudbattle.moves)

        if mode == self.modeOne:
            parMove  = Paragraph("Initial setup", styleNormal)
            board = self.createBoard(thudbattle,-1, self.cfg.sizeModeOne[1]-12) 
            parComment = Paragraph(cgi.escape(thudbattle.comment), styleNormal)
            arrBattle.append([parMove, board, parComment])

            index = 0
            for index in range(countMoves):
                parMove  = Paragraph(thudbattle.moves[index].str_save(), styleNormal)
                board = self.createBoard(thudbattle,index, self.cfg.sizeModeOne[1]-12) 
                parComment = Paragraph(cgi.escape(thudbattle.moves[index].comment), styleNormal)
                arrBattle.append([parMove, board, parComment])

            tabBattle = Table(arrBattle, [self.cfg.sizeModeOne[0]*mm, self.cfg.sizeModeOne[1]*mm, self.cfg.sizeModeOne[2]*mm])

        elif mode == self.modeTwo:
            parMove1  = Paragraph("Initial setup", styleNormal)
            board1 = self.createBoard(thudbattle,-1, self.cfg.sizeModeTwo[1]-12) 
            parComment1 = Paragraph(cgi.escape(thudbattle.comment), styleNormal)
            arrBattle.append([[parMove1, parComment1], board1, "", ""])

            index = 0
            for index in range(int((countMoves+1)/2)):
                parMove1  = Paragraph(thudbattle.moves[index*2].str_save(), styleNormal)
                board1 = self.createBoard(thudbattle,index*2, self.cfg.sizeModeTwo[1]-12) 
                parComment1 = Paragraph(cgi.escape(thudbattle.moves[index*2].comment), styleNormal)
                
                if (index*2+1) < countMoves:
                    parMove2  = Paragraph(thudbattle.moves[index*2+1].str_save(), styleNormal)
                    board2 = self.createBoard(thudbattle,index*2+1, self.cfg.sizeModeTwo[3]-12) 
                    parComment2 = Paragraph(cgi.escape(thudbattle.moves[index*2+1].comment), styleNormal)
                else:
                    parMove2 = Paragraph("", styleNormal);
                    board2 = Paragraph("", styleNormal);
                    parComment2 = Paragraph("", styleNormal);
 
                arrBattle.append([[parMove1, parComment1], board1, [parMove2, parComment2], board2])

            tabBattle = Table(arrBattle, [self.cfg.sizeModeTwo[0]*mm, self.cfg.sizeModeTwo[1]*mm, self.cfg.sizeModeTwo[2]*mm, self.cfg.sizeModeTwo[3]*mm])


        tabstyleBattle = TableStyle([ ('ALIGN',(0,0),(-1,-1), 'LEFT'),
                    ('VALIGN',(0,0),(-1,-1), 'TOP'),
                    ('INNERGRID',(0,0),(-1,-1), 0.25, (0,0,0))
                    ])
        tabBattle.setStyle(tabstyleBattle)

        arrPDF.append(tabBattle)
        docPDF = SimpleDocTemplate(filenamePDF)
        docPDF.build(arrPDF)
        return True
